@@ -26,19 +26,27 @@
 #include <config.h>
 
 #include "mutter-shaped-texture.h"
+#include "mutter-texture-tower.h"
 
 #include <clutter/clutter.h>
 #include <cogl/cogl.h>
 #include <string.h>
 
-
 static void mutter_shaped_texture_dispose (GObject *object);
 static void mutter_shaped_texture_finalize (GObject *object);
+static void mutter_shaped_texture_notify (GObject *object,
+					  GParamSpec *pspec);
 
 static void mutter_shaped_texture_paint (ClutterActor *actor);
 static void mutter_shaped_texture_pick (ClutterActor *actor,
 					const ClutterColor *color);
 
+static void mutter_shaped_texture_update_area (ClutterX11TexturePixmap *texture,
+                                               int                      x,
+                                               int                      y,
+                                               int                      width,
+                                               int                      height);
+
 static void mutter_shaped_texture_dirty_mask (MutterShapedTexture *stex);
 
 #ifdef HAVE_GLX_TEXTURE_PIXMAP
@@ -55,6 +63,7 @@ G_DEFINE_TYPE (MutterShapedTexture, mutter_shaped_texture,
 
 struct _MutterShapedTexturePrivate
 {
+  MutterTextureTower *paint_tower;
   CoglHandle mask_texture;
   CoglHandle material;
   CoglHandle material_unshaped;
@@ -62,7 +71,7 @@ struct _MutterShapedTexturePrivate
   CoglHandle material_workaround;
 #endif
 
-  GdkRegion *clip_region;
+  MetaRegion *clip_region;
 
   guint mask_width, mask_height;
 
@@ -74,13 +83,17 @@ mutter_shaped_texture_class_init (MutterShapedTextureClass *klass)
 {
   GObjectClass *gobject_class = (GObjectClass *) klass;
   ClutterActorClass *actor_class = (ClutterActorClass *) klass;
+  ClutterX11TexturePixmapClass *x11_texture_class = (ClutterX11TexturePixmapClass *) klass;
 
   gobject_class->dispose = mutter_shaped_texture_dispose;
   gobject_class->finalize = mutter_shaped_texture_finalize;
+  gobject_class->notify = mutter_shaped_texture_notify;
 
   actor_class->paint = mutter_shaped_texture_paint;
   actor_class->pick = mutter_shaped_texture_pick;
 
+  x11_texture_class->update_area = mutter_shaped_texture_update_area;
+
   g_type_class_add_private (klass, sizeof (MutterShapedTexturePrivate));
 }
 
@@ -93,6 +106,7 @@ mutter_shaped_texture_init (MutterShapedTexture *self)
 
   priv->rectangles = g_array_new (FALSE, FALSE, sizeof (XRectangle));
 
+  priv->paint_tower = mutter_texture_tower_new ();
   priv->mask_texture = COGL_INVALID_HANDLE;
 }
 
@@ -102,22 +116,26 @@ mutter_shaped_texture_dispose (GObject *object)
   MutterShapedTexture *self = (MutterShapedTexture *) object;
   MutterShapedTexturePrivate *priv = self->priv;
 
+  if (priv->paint_tower)
+    mutter_texture_tower_free (priv->paint_tower);
+  priv->paint_tower = NULL;
+
   mutter_shaped_texture_dirty_mask (self);
 
   if (priv->material != COGL_INVALID_HANDLE)
     {
-      cogl_material_unref (priv->material);
+      cogl_handle_unref (priv->material);
       priv->material = COGL_INVALID_HANDLE;
     }
   if (priv->material_unshaped != COGL_INVALID_HANDLE)
     {
-      cogl_material_unref (priv->material_unshaped);
+      cogl_handle_unref (priv->material_unshaped);
       priv->material_unshaped = COGL_INVALID_HANDLE;
     }
 #if 1 /* see comment in mutter_shaped_texture_paint */
   if (priv->material_workaround != COGL_INVALID_HANDLE)
     {
-      cogl_material_unref (priv->material_workaround);
+      cogl_handle_unref (priv->material_workaround);
       priv->material_workaround = COGL_INVALID_HANDLE;
     }
 #endif
@@ -138,6 +156,28 @@ mutter_shaped_texture_finalize (GObject *object)
   G_OBJECT_CLASS (mutter_shaped_texture_parent_class)->finalize (object);
 }
 
+static void
+mutter_shaped_texture_notify (GObject    *object,
+			      GParamSpec *pspec)
+{
+  if (G_OBJECT_CLASS (mutter_shaped_texture_parent_class)->notify)
+    G_OBJECT_CLASS (mutter_shaped_texture_parent_class)->notify (object, pspec);
+
+  /* It seems like we could just do this out of update_area(), but unfortunately,
+   * clutter_glx_texture_pixmap() doesn't call through the vtable on the
+   * initial update_area, so we need to look for changes to the texture
+   * explicitly.
+   */
+  if (strcmp (pspec->name, "cogl-texture") == 0)
+    {
+      MutterShapedTexture *stex = (MutterShapedTexture *) object;
+      MutterShapedTexturePrivate *priv = stex->priv;
+
+      mutter_texture_tower_set_base_texture (priv->paint_tower,
+					     clutter_texture_get_cogl_texture (CLUTTER_TEXTURE (stex)));
+    }
+}
+
 static void
 mutter_shaped_texture_dirty_mask (MutterShapedTexture *stex)
 {
@@ -151,10 +191,10 @@ mutter_shaped_texture_dirty_mask (MutterShapedTexture *stex)
       cogl_texture_get_gl_texture (priv->mask_texture,
                                    &mask_gl_tex, &mask_gl_target);
 
-      if (mask_gl_target == CGL_TEXTURE_RECTANGLE_ARB)
+      if (mask_gl_target == GL_TEXTURE_RECTANGLE_ARB)
         glDeleteTextures (1, &mask_gl_tex);
 
-      cogl_texture_unref (priv->mask_texture);
+      cogl_handle_unref (priv->mask_texture);
       priv->mask_texture = COGL_INVALID_HANDLE;
     }
 }
@@ -214,23 +254,23 @@ mutter_shaped_texture_ensure_mask (MutterShapedTexture *stex)
 
       cogl_texture_get_gl_texture (paint_tex, NULL, &paint_gl_target);
 
-      if (paint_gl_target == CGL_TEXTURE_RECTANGLE_ARB)
+      if (paint_gl_target == GL_TEXTURE_RECTANGLE_ARB)
         {
           GLuint tex;
 
           glGenTextures (1, &tex);
-          glBindTexture (CGL_TEXTURE_RECTANGLE_ARB, tex);
+          glBindTexture (GL_TEXTURE_RECTANGLE_ARB, tex);
           glPixelStorei (GL_UNPACK_ROW_LENGTH, tex_width);
           glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
           glPixelStorei (GL_UNPACK_SKIP_ROWS, 0);
           glPixelStorei (GL_UNPACK_SKIP_PIXELS, 0);
-          glTexImage2D (CGL_TEXTURE_RECTANGLE_ARB, 0,
+          glTexImage2D (GL_TEXTURE_RECTANGLE_ARB, 0,
                         GL_ALPHA, tex_width, tex_height,
                         0, GL_ALPHA, GL_UNSIGNED_BYTE, mask_data);
 
           priv->mask_texture
             = cogl_texture_new_from_foreign (tex,
-                                             CGL_TEXTURE_RECTANGLE_ARB,
+                                             GL_TEXTURE_RECTANGLE_ARB,
                                              tex_width, tex_height,
                                              0, 0,
                                              COGL_PIXEL_FORMAT_A_8);
@@ -260,13 +300,36 @@ mutter_shaped_texture_paint (ClutterActor *actor)
   ClutterActorBox alloc;
   CoglHandle material;
 
-  if (priv->clip_region && gdk_region_empty (priv->clip_region))
+  if (priv->clip_region && meta_region_is_empty (priv->clip_region))
     return;
 
   if (!CLUTTER_ACTOR_IS_REALIZED (CLUTTER_ACTOR (stex)))
     clutter_actor_realize (CLUTTER_ACTOR (stex));
 
-  paint_tex = clutter_texture_get_cogl_texture (CLUTTER_TEXTURE (stex));
+  /* If mipmaps are supported, then the texture filter quality will
+   * still be HIGH here. In that case we just want to use the base
+   * texture. If mipmaps are not support then
+   * on_glx_texture_pixmap_pre_paint() will have reset the texture
+   * filter quality to MEDIUM, and we should use the MutterTextureTower
+   * mipmap emulation.
+   *
+   * http://bugzilla.openedhand.com/show_bug.cgi?id=1877 is an RFE
+   * for a better way of handling this.
+   *
+   * While it would be nice to have direct access to the 'can_mipmap'
+   * boolean in ClutterGLXTexturePixmap, since since MutterTextureTower
+   * creates the scaled down images on demand there is no substantial
+   * overhead from doing the work to create and update the tower and
+   * not using it, other than the memory allocated for the MutterTextureTower
+   * structure itself.
+   */
+  if (clutter_texture_get_filter_quality (CLUTTER_TEXTURE (stex)) == CLUTTER_TEXTURE_QUALITY_HIGH)
+    paint_tex = clutter_texture_get_cogl_texture (CLUTTER_TEXTURE (stex));
+  else
+    paint_tex = mutter_texture_tower_get_paint_texture (priv->paint_tower);
+
+  if (paint_tex == COGL_INVALID_HANDLE)
+    return;
 
   tex_width = cogl_texture_get_width (paint_tex);
   tex_height = cogl_texture_get_height (paint_tex);
@@ -274,9 +337,6 @@ mutter_shaped_texture_paint (ClutterActor *actor)
   if (tex_width == 0 || tex_height == 0) /* no contents yet */
     return;
 
-  if (paint_tex == COGL_INVALID_HANDLE)
-    return;
-
   if (priv->rectangles->len < 1)
     {
       /* If there are no rectangles use a single-layer texture */
@@ -327,7 +387,7 @@ mutter_shaped_texture_paint (ClutterActor *actor)
 #     define MAX_RECTS 16
 
       /* Would be nice to be able to check the number of rects first */
-      gdk_region_get_rectangles (priv->clip_region, &rects, &n_rects);
+      meta_region_get_rectangles (priv->clip_region, &rects, &n_rects);
       if (n_rects > MAX_RECTS)
 	{
 	  g_free (rects);
@@ -335,24 +395,34 @@ mutter_shaped_texture_paint (ClutterActor *actor)
 	}
       else
 	{
-	  float coords[MAX_RECTS * 8];
+	  float coords[8];
+          float x1, y1, x2, y2;
+
 	  for (i = 0; i < n_rects; i++)
 	    {
 	      GdkRectangle *rect = &rects[i];
 
-	      coords[i * 8 + 0] = rect->x;
-	      coords[i * 8 + 1] = rect->y;
-	      coords[i * 8 + 2] = rect->x + rect->width;
-	      coords[i * 8 + 3] = rect->y + rect->height;
-	      coords[i * 8 + 4] = rect->x / (alloc.x2 - alloc.x1);
-	      coords[i * 8 + 5] = rect->y / (alloc.y2 - alloc.y1);
-	      coords[i * 8 + 6] = (rect->x + rect->width) / (alloc.x2 - alloc.x1);
-	      coords[i * 8 + 7] = (rect->y + rect->height) / (alloc.y2 - alloc.y1);
-	    }
+	      x1 = rect->x;
+	      y1 = rect->y;
+	      x2 = rect->x + rect->width;
+	      y2 = rect->y + rect->height;
+
+              coords[0] = rect->x / (alloc.x2 - alloc.x1);
+	      coords[1] = rect->y / (alloc.y2 - alloc.y1);
+	      coords[2] = (rect->x + rect->width) / (alloc.x2 - alloc.x1);
+	      coords[3] = (rect->y + rect->height) / (alloc.y2 - alloc.y1);
+
+              coords[4] = coords[0];
+              coords[5] = coords[1];
+              coords[6] = coords[2];
+              coords[7] = coords[3];
+
+              cogl_rectangle_with_multitexture_coords (x1, y1, x2, y2,
+                                                       &coords[0], 8);
+            }
 
 	  g_free (rects);
 
-	  cogl_rectangles_with_texture_coords (coords, n_rects);
 	  return;
 	}
     }
@@ -406,6 +476,22 @@ mutter_shaped_texture_pick (ClutterActor *actor,
     }
 }
 
+static void
+mutter_shaped_texture_update_area (ClutterX11TexturePixmap *texture,
+                                   int                      x,
+                                   int                      y,
+                                   int                      width,
+                                   int                      height)
+{
+  MutterShapedTexture *stex = (MutterShapedTexture *) texture;
+  MutterShapedTexturePrivate *priv = stex->priv;
+
+  CLUTTER_X11_TEXTURE_PIXMAP_CLASS (mutter_shaped_texture_parent_class)->update_area (texture,
+                                                                                      x, y, width, height);
+
+  mutter_texture_tower_update_area (priv->paint_tower, x, y, width, height);
+}
+
 ClutterActor *
 mutter_shaped_texture_new (void)
 {
@@ -471,7 +557,7 @@ mutter_shaped_texture_add_rectangles (MutterShapedTexture *stex,
  */
 void
 mutter_shaped_texture_set_clip_region (MutterShapedTexture *stex,
-				       GdkRegion           *clip_region)
+				       MetaRegion          *clip_region)
 {
   MutterShapedTexturePrivate *priv;
 
@@ -481,7 +567,7 @@ mutter_shaped_texture_set_clip_region (MutterShapedTexture *stex,
 
   if (priv->clip_region)
     {
-      gdk_region_destroy (priv->clip_region);
+      meta_region_destroy (priv->clip_region);
       priv->clip_region = NULL;
     }
 
