@@ -68,6 +68,7 @@
 #include <fcntl.h>
 #include <locale.h>
 #include <time.h>
+#include <unistd.h>
 
 #include <clutter/clutter.h>
 #include <clutter/x11/clutter-x11.h>
@@ -77,12 +78,6 @@
 #include "compositor/mutter-plugin-manager.h"
 #endif
 
-#include <X11/cursorfont.h>
-
-#ifdef HAVE_XCURSOR
-#include <X11/Xcursor/Xcursor.h>
-#endif
-
 /**
  * The exit code we'll return to our parent process when we eventually die.
  */
@@ -130,11 +125,13 @@ log_handler (const gchar   *log_domain,
 static void
 version (void)
 {
+  const int latest_year = 2010;
+
   g_print (_("mutter %s\n"
-             "Copyright (C) 2001-2008 Havoc Pennington, Red Hat, Inc., and others\n"
+             "Copyright (C) 2001-%d Havoc Pennington, Red Hat, Inc., and others\n"
              "This is free software; see the source for copying conditions.\n"
              "There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"),
-           VERSION);
+           VERSION, latest_year);
   exit (0);
 }
 
@@ -232,6 +229,7 @@ typedef struct
   gboolean sync;
   gboolean composite;
   gboolean no_composite;
+  gboolean no_force_fullscreen;
   gboolean no_tab_popup;
   gchar *introspect;
 } MetaArguments;
@@ -310,6 +308,12 @@ meta_parse_options (int *argc, char ***argv,
       N_("Turn compositing off"),
       NULL
     },
+    {
+      "no-force-fullscreen", 0, COMPOSITE_OPTS_FLAGS, G_OPTION_ARG_NONE,
+      &my_args.no_force_fullscreen,
+      N_("Don't make fullscreen windows that are maximized and have no decorations"),
+      NULL
+    },
     {
       "mutter-plugins", 0, 0, G_OPTION_ARG_STRING,
       &my_args.mutter_plugins,
@@ -337,6 +341,7 @@ meta_parse_options (int *argc, char ***argv,
   ctx = g_option_context_new (NULL);
   g_option_context_add_main_entries (ctx, options, "mutter");
   g_option_context_add_group (ctx, clutter_get_option_group_without_init ());
+  g_option_context_add_group (ctx, cogl_get_option_group ());
 
   if (!g_option_context_parse (ctx, argc, argv, &error))
     {
@@ -442,37 +447,28 @@ meta_finalize (void)
   meta_session_shutdown ();
 }
 
+static int sigterm_pipe_fds[2] = { -1, -1 };
+
 static void
 sigterm_handler (int signum)
 {
-  meta_finalize ();
-
-  exit (meta_exit_code);
-}
-
-static guint sigchld_signal_id = 0;
-
-static void
-sigchld_handler (int signum, siginfo_t *info, void *context)
-{
-  int stat;
-
-  if (info->si_code == CLD_EXITED)
+  if (sigterm_pipe_fds[1] >= 0)
     {
-      g_signal_emit (sigchld_nexus, sigchld_signal_id, 0,
-                     info->si_status,
-                     GINT_TO_POINTER (info->si_pid));
+      int dummy;
+
+      dummy = write (sigterm_pipe_fds[1], "", 1);
+      close (sigterm_pipe_fds[1]);
+      sigterm_pipe_fds[1] = -1;
     }
-
-  g_signal_handlers_disconnect_matched (sigchld_nexus,
-                                        G_SIGNAL_MATCH_DATA,
-                                        sigchld_signal_id,
-                                        0, NULL, NULL,
-                                        GINT_TO_POINTER (info->si_pid));
-
-  waitpid (info->si_pid, &stat, WNOHANG);
 }
-    
+
+static gboolean
+on_sigterm (void)
+{
+  meta_quit (META_EXIT_SUCCESS);
+  return FALSE;
+}
+
 /**
  * This is where the story begins. It parses commandline options and
  * environment variables, sets up the screen, hands control off to
@@ -495,8 +491,9 @@ main (int argc, char **argv)
     "Pango", "GLib-GObject", "GThread"
   };
   guint i;
+  GIOChannel *channel;
   GOptionContext *ctx;
-  
+
   if (!g_thread_supported ())
     g_thread_init (NULL);
   
@@ -518,29 +515,21 @@ main (int argc, char **argv)
                 g_strerror (errno));
 #endif
 
+  if (pipe (sigterm_pipe_fds) != 0)
+    g_printerr ("Failed to create SIGTERM pipe: %s\n",
+                g_strerror (errno));
+
+  channel = g_io_channel_unix_new (sigterm_pipe_fds[0]);
+  g_io_channel_set_flags (channel, G_IO_FLAG_NONBLOCK, NULL);
+  g_io_add_watch (channel, G_IO_IN, (GIOFunc) on_sigterm, NULL);
+  g_io_channel_set_close_on_unref (channel, TRUE);
+  g_io_channel_unref (channel);
+
   act.sa_handler = &sigterm_handler;
   if (sigaction (SIGTERM, &act, NULL) < 0)
     g_printerr ("Failed to register SIGTERM handler: %s\n",
 		g_strerror (errno));
 
-  sigchld_nexus = g_object_new (META_TYPE_NEXUS, NULL);
-
-  sigchld_signal_id =
-    g_signal_new ("sigchld", META_TYPE_NEXUS,
-                  G_SIGNAL_RUN_LAST,
-                  0, NULL, NULL,
-                  g_cclosure_marshal_VOID__UINT_POINTER,
-                  G_TYPE_NONE,
-                  2,
-                  G_TYPE_UINT, G_TYPE_POINTER);
-  
-  act.sa_flags = SA_NOCLDSTOP | SA_SIGINFO;
-  act.sa_handler = SIG_DFL;
-  act.sa_sigaction = &sigchld_handler;
-  if (sigaction (SIGCHLD, &act, NULL) < 0)
-    g_printerr ("Failed to register SIGCHLD handler: %s\n",
-		g_strerror (errno));
-
   if (g_getenv ("MUTTER_VERBOSE"))
     meta_set_verbose (TRUE);
   if (g_getenv ("MUTTER_DEBUG"))
@@ -597,7 +586,7 @@ main (int argc, char **argv)
            * is initialized at this point, and we don't plan to run any real
            * plugin code.
            */
-          MutterPluginManager *mgr = mutter_plugin_manager_new (NULL);
+          MutterPluginManager *mgr = mutter_plugin_manager_get_default ();
           if (!mutter_plugin_manager_load (mgr))
             g_critical ("failed to load plugins");
         }
@@ -632,36 +621,6 @@ main (int argc, char **argv)
    */
   meta_clutter_init (ctx, &argc, &argv);
 
-  {
-    /*
-     * Set busy cursor as soon as we can (at this there is no MetaDisplay or
-     * MetaScreen object, so we have to do this the xlib way.
-     *
-     * Metacity will reset the cursor as soon as it creates the default
-     * MetaScreen object. Plugins might want to further control the cursor
-     * in their constructor.
-     */
-    Display *xdpy = meta_ui_get_display ();
-    Cursor   xc;
-
-#ifdef HAVE_XCURSOR
-    /*
-     * Obviously, this is moblin specific; we want the correct theme
-     * applied from the start, and we cannot query the prefs yet, but the
-     * prefs take a significant amount of time to query, and we do not want
-     * the uggly X watch cursor there at all.
-     */
-    XcursorSetTheme (xdpy, "moblin");
-    XcursorSetDefaultSize (xdpy, 24);
-#endif
-
-    xc = XCreateFontCursor (xdpy, XC_watch);
-
-    XDefineCursor (xdpy, RootWindow (xdpy, 0), xc);
-    XSync (xdpy,False);
-    XFreeCursor (xdpy, xc);
-  }
-
   g_option_context_free (ctx);
 
   /* must be after UI init so we can override GDK handlers */
@@ -669,7 +628,6 @@ main (int argc, char **argv)
 
   /* Load prefs */
   meta_prefs_init ();
-
   meta_prefs_add_listener (prefs_changed_callback, NULL);
 
 
@@ -753,6 +711,9 @@ main (int argc, char **argv)
   if (meta_args.composite || meta_args.no_composite)
     meta_prefs_set_compositing_manager (meta_args.composite);
 
+  if (meta_args.no_force_fullscreen)
+    meta_prefs_set_force_fullscreen (FALSE);
+
   if (meta_args.no_tab_popup)
     {
       meta_prefs_override_no_tab_popup (TRUE);
@@ -760,17 +721,7 @@ main (int argc, char **argv)
 
   if (!meta_display_open ())
     meta_exit (META_EXIT_ERROR);
-
-  /*
-   * This seems the first time we can set up the cursor properly. The display
-   * seems to set up the theme at the point it opens, but the cursor size
-   * for individual screens seems wrong unless we update it here.
-   *
-   * TODO -- investigate further to avoid unnecessary X round trips.
-   */
-  meta_display_set_cursor_theme (meta_prefs_get_cursor_theme (),
-                                 meta_prefs_get_cursor_size ());
-
+  
   g_main_loop_run (meta_main_loop);
 
   meta_finalize ();
