@@ -32,6 +32,7 @@
 #include "theme.h"
 
 #include "inlinepixbufs.h"
+#include "gdk-compat.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -475,7 +476,7 @@ get_cmap (GdkPixmap *pixmap)
 
   /* Be sure we aren't going to blow up due to visual mismatch */
   if (cmap &&
-      (gdk_colormap_get_visual (cmap)->depth !=
+      (gdk_visual_get_depth (gdk_colormap_get_visual (cmap)) !=
        gdk_drawable_get_depth (pixmap)))
     {
       cmap = NULL;
